#!/usr/bin/python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os
import os.path
import subprocess
import gobject

from tacix.util import *

class TXSound:
    def __init__ (self, sessionid):
        self.txsession = get_session(sessionid)
        self.txpath = TXPath(sessionid)
        self.logger = self.txpath.get_logger("txsound")

        self.signal = None
        self.options = self.txsession.GetProperties()
        self.resource = None
        self.counter = None

    def watch_agent_state(self, state):
        self.logger.info("Sound watcher state is %s" % state)
        if state == 'running':
            if self.options is None:
                self.options = self.txsession.GetProperties()
            if 'mediahelper' in self.options:
                gobject.timeout_add(1000, self.init_sound)
        elif state == 'terminating' or state == 'suspending':
            if 'mediahelper' in self.options:
                self.cleanup_sound()
        elif state == 'terminated' or state == 'suspended':
            if 'mediahelper' in self.options:
                self.cleanup_sound()
            self.options = None

    def setup_signal(self):
        self.logger.debug('Adding watcher for TXSound')
        if self.options is None:
            self.logger.debug('Options is None')
            return
        self.signal = self.txsession.connect_to_signal("StateChanged", self.watch_agent_state, dbus_interface=SESSION_INTERFACE)
        if self.txsession.GetState() == "running":
            self.init_sound()
        elif state == 'terminating' or state == 'suspending':
            self.cleanup_sound()
        elif state == 'terminated' or state == 'suspended':
            self.cleanup_sound()

    def init_sound(self):
        self.logger.debug('init_sound')
        if self.options is None:
            self.logger.debug('Options is None')
            return False
        if self.options['mediahelper'] == 'esd':
            self.logger.debug('Adding sink to pulseaudio')
            sound_port = str(int(self.options['display']) + 7000)
            cmd = 'pactl load-module module-esound-sink sink_name=nx server="127.0.0.1:%s"' % sound_port
            #cmd = 'pactl load-module module-tunnel-sink sink_name=nx server="127.0.0.1:%s"' % sound_port
            self.logger.debug(cmd)
            pa_process = subprocess.Popen(cmd,
                    shell=True, stdout=subprocess.PIPE, stderr = None)
            out, err = pa_process.communicate()
            ret = pa_process.wait()
            if ret == 0:
                self.logger.debug('PulseAudio configured with module n %s' % out)
                cmd_process = subprocess.Popen('pacmd', shell=True, stdout=None, stderr=None, stdin=subprocess.PIPE, close_fds=True)
                cmd_process.communicate('set-default-sink nx\n')
                cmd_process.stdin.close()
                cmd_process.wait()

                self.resource = out.strip()
                self.logger.debug(out)
                self.txsession.Notify('TaciX sound support', 'PulseAudio redirected to remote server')
                self.counter = None
                return False

            self.logger.debug('PulseAudio configuration failed')
            if self.counter is None:
                self.counter = 30
            self.counter = self.counter - 1
            if self.counter == 0:
                self.counter = None
                self.txsession.Notify('TaciX sound support', 'Failed to redirect PulseAudio remote server')
                return False
            return True
        self.logger.debug('Sound type not defined')
        return False

    def cleanup_sound(self):
        self.logger.debug('cleanup_sound')
        if self.options is None:
            self.logger.debug('Options is None')
            return
        if self.options['mediahelper'] == 'esd':
            if self.resource is None:
                self.logger.debug('Sound not configured')
                return
            pa_process = subprocess.call('pactl unload-module %s"' % self.resource,
                    shell=True, stdout=subprocess.PIPE)
            if ret == 0:
                self.logger.debug('PulseAudio module removed')
                return
            self.logger.debug('Failed removing PulseAudio module')
            self.resource = None

