#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import sys
from tacix.freenx.FreeNXProtocol import FreeNXProtocol

LISTSESSION_COLUMNS = [
        ["Display", 7, "display"],
        ["Type", 16, "type"],
        ["Session ID", 32, "ck_session_id"],
        ["Options", 8, "options"],
        ["Depth", -5, "depth"],
        ["Screen", 14, "geometry"],
        ["Status", 11, "state"],
        ["Session Name", 30, 'session']]

class FreeNXNodeProtocol(FreeNXProtocol):
    def __init__(self):
        # Needed attributes
        self.sessionid = ''

    def exit(self):
        self.say_bye()

    def check(self):
        self.say_hello()
        self.say_finished()
        self.say_bye()

    # Needed methods
    def get_property(self, property):
        return ''

    # Output functions
    def send_client_server(self, line):
        self.send_client(line + "\n")

    def say_hello(self):
        sys.stdout.write("NX> 1000 NXNODE - Version 3.3.0-1-BZR OS (GPL, using backend: 3.3.0)\n")
        sys.stdout.flush()

    def say_bye(self):
        self.send_client_server( "NX> 1001 Bye." )

    def say_finished(self):
        self.send_client_server( "NX> 716 finished" )

    def say_suspending(self):
        self.send_client_server("NX> 716 Suspending session %s on user request." % self.sessionid)

    def say_session_resume_failed(self):
        self.send_client_server('NX> 596 Error: Session failed.')

    def say_session_resumed(self):
        self.send_client_server( 'NX> 718 Session restore succeded' )

    def say_suspend_failed_not_found(self):
        self.send_client_server("NX> 1004 Error: Could not resume session. nxagent process could not be found.")

    def say_terminating(self):
        self.send_client_server("NX> 716 Terminating session %s on user request.", self.sessionid)

    def say_slave_started(self):
        self.send_client_server("NX> 716 Slave mode started successfully.")

    def say_closed(self):
        self.send_client_server("NX> 1009 Session status: closed")

    def say_connected(self):
        self.send_client_server( "NX> 700 Session id: %s" % self.get_property('id') )
        self.send_client_server( "NX> 705 Session display: %s" % self.get_property('display') )
        self.send_client_server( "NX> 703 Session type: %s" % self.get_property('type') )
        self.send_client_server( "NX> 701 Proxy cookie: %s" % self.get_property('cookie') )
        self.send_client_server( "NX> 702 Proxy IP: %s" % self.get_property('proxyip') )
        self.send_client_server( "NX> 706 Agent cookie: %s" % self.get_property('cookie') )
        self.send_client_server( "NX> 704 Session cache: %s" % self.get_property('cache') )
        self.send_client_server( "NX> 707 SSL tunneling: %s" % self.get_property('encryption') )

    def say_error(self):
        self.send_client_server( "NX> 1004 Error: NX Agent exited with exit status 1. To troubleshoot set SESSION_LOG_CLEAN=0 "
               "in node.conf and investigate \"$USER_FAKE_HOME/.nx/F-C-$sess_id/session\". You might also "
               "want to try: ssh -X myserver; $PATH_BIN/nxnode --agent to test the basic functionality."
               " Session log follows:")

    def say_waiting(self):
        self.send_client_server('NX> 710 Session status: running')
        self.send_client_server('NX> 1002 Commit')
        self.send_client_server('NX> 1006 Session status: running')

    def say_shadowsession_header(self, line):
        sys.stdout.write('NX> 127 Sessions list for reconnect:\n')
        sys.stdout.write('\n')
        self.send_client_server("Display Type             Session ID                       Options  Depth Screensize     Available Session Name")
        self.send_client_server("------- ---------------- -------------------------------- -------- ----- -------------- --------- ----------------------")

    def say_shadowsessions_sessions(self, options):
        # Print sessions
        self.send_client_server('%(display)-7s %(type)-16s %(session)-32s %(cli_opts)-8s %(depth)5s %(screen)-14s %(available)-11s %(session)s'
                % options)

    def say_listsessions_header(self, options):
        sys.stdout.write('NX> 127 Sessions list of user \'%s\' for reconnect:\n' % options['user'])
        sys.stdout.write('\n')

        # Print headers
        colsh = []
        colsd = []
        for header,width,param in LISTSESSION_COLUMNS:
            colsd.append("-" * abs(width))
            if width >= 0:
                text = header.ljust(width)
            else:
                text = header.rjust(abs(width))
            colsh.append(text)
        sys.stdout.write(" ".join(colsh) + '\n')
        sys.stdout.write(" ".join(colsd) + '\n')
        sys.stdout.flush()

    def say_listsessions_sessions(self, options):
        # Print sessions
        sys.stdout.write('%(display)-7s %(type)-16s %(ck_session_id)-32s %(cli_opts)-8s %(depth)5s %(screen)-14s %(state)-11s %(session)-30s\n'
                % options)

        sys.stdout.flush()

    def say_listsessions_footer(self, options):
        sys.stdout.write('\n')
        sys.stdout.write('\n')
        if 'user' in options:
            sys.stdout.write('NX> 148 Server capacity: not reached for user: %s\n' % options['user'])
        else:
            sys.stdout.write('NX> 148 Server capacity: not reached for user\n')
        sys.stdout.write('NX> 105 ')
        sys.stdout.flush()

