#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os


from ConfigParser import ConfigParser

default_options = {}

class Config:
    def __init__(self, f='/etc/tacix/tacix.conf'):
        self.parser = ConfigParser()
        self.filename = f
        self.load_config()
        self.permissive = True
        self.allowed_options = {
                  # Enum values
                  'link' : ['modem', 'isdn', 'adsl', 'wan', 'lan'],
                  'mediahelper' : ['pulse', 'esd', 'artsd'],
                  'type' : ['unix-gnome', 'unix-kde', 'unix-cde', 'unix-xdm',
                      'unix-console', 'unix-default', 'unix-application',
                      'windows', 'vnc', 'shadow'],
                  'client' : ['linux'],
                  'clipboard' : ['both'],
                  'cache' : ['4M', '8M', '16M', '32M', '64M', '128M'], # In memory
                  'images' : ['0M', '32M', '64M', '128M', '256M', '512M'], # On disk

                  # Boolean values
                  'strict' : ['0', '1'],
                  'persistent' : ['1', '0'],
                  
                  # Client options
                  'backingstore' : ['1', '0'],
                  'encryption' : ['1', '0'],
                  'shmem' : ['1', '0'],
                  'shpix' : ['1', '0'], # Needs shmem enabled
                  'composite' : ['1', '0'],
                  'media' : ['0', '1'],
                  'render' : ['1', '0'],
                  'fullscreen' : ['0', '1'],
                  'virtualdesktop' : ['1', '0'],
                  'rootless' : ['0', '1'], # Needs virtualdesktop = 0, 0 = Disable X agent encoding
                  'taint' : ['1', '0'], # Needs rootless = 0, 0 = Disable taints of X replies
                  'stream' : ['1', '0'], # 0 = Disable zlib stream compression
                  'defer' : ['0', '1'], # 1 = Disable deferred screen updates

                  'depth' : '24',
                  'screen' : '1024x768',

                  'application' : '',

                  # String client options
                  'imagecompressionmethod' : '', # None = ?, -1 = jpeg, 0 = None, 1 = jpeg + custom compression, 2 = rgb, 3 = jpeg + rgb, 4 = jpeg + rgb + custom compression
                  'imagecompressionlevel' : '', # Needs imagecompression 1 or 4, 0-9

                  # String values
                  'cleanup' : '3', # Timeout to kill wathdog process default is 3, 0 don't timeout
                  'display' : '1000',
                  'session' : '',
                  'keyboard' : '',
                  'screeninfo' : '',
                  'geometry' : '', # 800x600

                  'txsession_pid' : '',

                  # Shadow properties
                  'shadowed_id' : '',
                  'shadowdisplay' : '',
                  'authority_file' : '',

                  # Windows and vnc properties
                  'agent_password' : '',
                  'agent_server' : '',
                  # Windows properties
                  'agent_domain' : '',
                  'agent_user' : ''
                  }
        self.allowed_options.update(self.to_dict())


    def load_config(self):
        try:
            self.parser.read(self.filename)
        except Exception, e:
            pass

    def to_dict(self):
        dict = {}
        try:
            for opt in self.parser.options('tacix'):
                opts = self.parser.get('tacix', opt).split(' ')
                if len(opts) > 1:
                    dict[opt] = opts
                else:
                    dict[opt] = opts[0]
        except Exception,e:
            pass
        return dict

    def get_allowed_options(self):
        return self.allowed_options

    def get_allowed_value( self, property, value ):
        if self.permissive:
            return value
        new_value = None

        if property in self.allowed_options:
            avalues = self.allowed_options[property]
            if type(avalues) == str:
                new_value = avalues

            elif value not in avalues:
                new_value = avalues[0]

        if permissive:
            self.logger.debug('Permissive method allowed property %s with value %s instead of changing it to %s' %\
                    (property, value, new_value))
            return value

        self.logger.debug('Property %s with value %s not allowed, changing it to %s' %\
                (property, value, new_value))
        return new_value

    def __getitem__(self, name):
        s = self.parser.get('tacix', name)
        return s

    def get_default_options(self):
        allowed_options = self.get_allowed_options()

        if len(default_options) == 0:
            for key in allowed_options:
                value = allowed_options[key]
                if value is not None and type(value) is not str:
                    default_options[key] = value[0]
                elif type(value) is str and value != '':
                    default_options[key] = value

        ret = {}
        ret.update(default_options)
        return ret

    def get_default_option( self, property ):
        defopt = self.get_default_options()
        if property in defopt:
            return defopt[property]
        return None


if __name__ == "__main__":
    config = Config()
    print config.to_dict()
