#!/usr/bin/python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import socket
import os

def find_next_available_display( min_display ):
    display = min_display
    while not is_display_available( display ):
        display = int(display) + 1
    return display

def get_lock_files ( display ):
    return ["/tmp/.X" + str( display ) + "-lock", "/tmp/.nX" + str( display ) + "-lock",
            "/tmp/.X11-unix/X" + str( display ), "/tmp/.nX" + str( display ) + "-lock.XXXXXXXXX"]

def get_used_ports ( display ):
    return [int(display) + 3000, int(display) + 4000, int(display) + 6000, int(display) + 7000, int(display) + 9000]

def is_display_available( display ):
    for path in get_lock_files( display ):
        if os.path.lexists( path ):
            #print('File %s exists' % path)
            return False

    for port in get_used_ports( display ):
        if not is_port_available( port ):
            #print('Port %s is not available' % str(port))
            return False

    return True

def is_port_available( port ):
    try:
        HOST = ''                 # Symbolic name meaning the local host
        s = socket.socket()
        s.bind((HOST, port))
        s.listen(1)
        s.close( )
        return True
    except socket.error:
        return False

